<?php
/**
 * @package        JFBConnect
 * @copyright (C) 2009-2013 by Source Coast - All rights reserved
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.controller');
jimport('sourcecoast.utilities');

class JFBConnectControllerLogin extends JControllerLegacy
{

    function display($cachable = false, $urlparams = false)
    {
        JFactory::getApplication()->redirect('index.php');
    }

    function login($provider)
    {
        // Let the provider do anything it wants before we try to login.
        $provider->onBeforeLogin();

        $loginRegisterModel = $this->getModel('LoginRegister', 'JFBConnectModel');

        // Set a cookie to prevent auto-logging in for the remainder of the session time
        $config = JFactory::getConfig();
        $lifetime = $config->get('lifetime', 15);
        setcookie('jfbconnect_autologin_disable', 1, time() + ($lifetime * 60));
        // Not a perfect solution, but fixes autologin loops..

        $app = JFactory::getApplication();
        $providerUserId = $provider->getProviderUserId();

        if (!$providerUserId)
        { # Facebook isn't returning information about this user.  Redirect them.
            $app->enqueueMessage(JText::sprintf('COM_JFBCONNECT_UNABLE_TO_RETRIEVE_USER', ucfirst($provider->name)));
            $app->redirect('index.php');
        }

        $userMapModel = JFBCFactory::usermap();

        $jUser = JFactory::getUser();
        if (!$jUser->guest) # User is already logged into Joomla. Update their facebook mapping
        {
            SCSocialUtilities::clearJFBCNewMappingEnabled();
            if ($userMapModel->map($jUser->get('id'), $providerUserId, $provider->name, $provider->getToken()))
                $app->enqueueMessage(JText::sprintf('COM_JFBCONNECT_MAP_USER_SUCCESS', ucfirst($provider->name)));
            else
                $app->enqueueMessage(JText::sprintf('COM_JFBCONNECT_MAP_USER_FAIL', ucfirst($provider->name)));

            $return = $loginRegisterModel->getLoginRedirect($provider);
            $app->redirect($return);
        }

        // They're not loged in. Check if they have a Joomla user and log that user in. If not, create them one
        $jUserId = $userMapModel->getJoomlaUserId($providerUserId, $provider->name);

        # Check if automatic email mapping is allowed, and see if that email is registered
        # AND the Facebook user doesn't already have a Joomla account
        if (!$provider->initialRegistration && JFBCFactory::config()->getSetting('facebook_auto_map_by_email') && !$jUserId)
        {
            $profile = $provider->profile->fetchProfile($providerUserId, array('email'));

            $providerEmail = $profile->get('email', null);
            if ($providerEmail != null)
            {
                $jUserEmailId = $userMapModel->getJoomlaUserIdFromEmail($providerEmail);
                if (!empty($jUserEmailId))
                {
                    // Found a user with the same email address
                    // do final check to make sure there isn't a FB account already mapped to it
                    $tempId = $userMapModel->getProviderUserId($jUserEmailId, $provider->name);
                    if (!$tempId)
                    {
                        SCSocialUtilities::clearJFBCNewMappingEnabled();
                        if ($userMapModel->map($jUserEmailId, $providerUserId, $provider->name, $provider->getToken()))
                        {
                            $app->enqueueMessage(JText::sprintf('COM_JFBCONNECT_MAP_USER_SUCCESS', ucfirst($provider->name)));
                            $jUserId = $jUserEmailId; // Update the temp jId so that we login below
                        }
                        else
                            $app->enqueueMessage(JText::sprintf('COM_JFBCONNECT_MAP_USER_FAIL', ucfirst($provider->name)));

                    }
                }
            }
        }

        if (JFBCFactory::config()->getSetting('create_new_users') && $jUserId == null)
        { # User not in system, create new users setting enabled. Redirect to the login/register form
            $return = $loginRegisterModel->getLoginRedirect($provider);
            $app->redirect(JRoute::_('index.php?option=com_jfbconnect&view=loginregister&provider=' . $provider->name . '&return=' . base64_encode($return), false));
        }

        // Check if no mapping, and Automatic Registration is set. If so, create the new user.
        if (!$jUserId && !JFBCFactory::config()->getSetting('create_new_users'))
        { # User is not in system, should create their account automatically
            if ($loginRegisterModel->autoCreateUser($providerUserId, $provider))
                $jUserId = $userMapModel->getJoomlaUserId($providerUserId, $provider->name);
        }

        // If no jUserId at this point, there won't be one. We've failed.
        if (!$jUserId)
            return false;

        $jUser = JUser::getInstance($jUserId);

        $loginSuccess = false;
        // Try to log the user, but not if blocked and initial registration (then there will be a pretty message on how to activate)
        if (!$provider->initialRegistration || ($jUser->get('block') == 0 && $provider->initialRegistration))
        {
            $options = array('silent' => 1, 'provider' => $provider, 'provider_user_id' => $providerUserId); // Disable other authentication messages
            $loginSuccess = $app->login(array('username' => $provider->appId, 'password' => $provider->secretKey), $options);
        }

        if ($loginSuccess)
        {
            // lets update the user's access token with whatever we just received
            $jUser = JFactory::getUser();
            $userMapModel->map($jUser->get('id'), $providerUserId, $provider->name, $provider->getToken());

            if (!$provider->initialRegistration)
            {
                $args = array($provider->name, $jUser->get('id'), $providerUserId);
                $app->triggerEvent('socialProfilesOnLogin', $args);
            }
        }

        JFactory::getApplication()->setUserState('com_jfbconnect.' . $provider->name . '.checkForNewMapping', null);
        $allProviders = JFBCFactory::getAllProviders();
        foreach ($allProviders as $p)
            $p->checkNewMapping();

        $redirect = $loginRegisterModel->getLoginRedirect($provider);

        $app->redirect($redirect);
    }

    /*  Not ready for primetime yet. The setInitialRegistration causes issues.
    function updateProfile()
    {
        $jUser = JFactory::getUser();
        $jfbcLibrary = JFBConnectFacebookLibrary::getInstance();
        $jfbcLibrary->setInitialRegistration();
        $fbUserId = $jfbcLibrary->getMappedFbUserId();
        $args = array($jUser->get('id'), $fbUserId);

        $app = JFactory::getApplication();
        JPluginHelper::importPlugin('jfbcprofiles');
        $app->triggerEvent('scProfilesImportProfile', $args);
        $app->enqueueMessage('Profile Imported!');
        $app->redirect('index.php');
    }*/

    /*
* Send message to user notifying them of the new account and if they have to activate.
* If default Mail email and name are not set, grab it from a super admin in the DB.
*/

}
