<?php
/**
 * @package        JFBConnect
 * @copyright (C) 2009-2013 by Source Coast - All rights reserved
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');


    JLoader::register('JGoogleAuth', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/auth.php');
    JLoader::register('JGoogleData', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/data.php');
    JLoader::register('JGoogle', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/google.php');
    JLoader::register('JGoogleAuthOauth2', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/auth/oauth2.php');
    JLoader::register('JGoogleDataPlus', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/data/plus.php');
    JLoader::register('JGoogleDataPlusActivities', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/data/plus/activities.php');
    JLoader::register('JGoogleDataPlusActivities', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/data/plus/activities.php');
    JLoader::register('JGoogleDataPlusComments', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/data/plus/comments.php');
    JLoader::register('JGoogleDataPlusPeople', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/google/data/plus/people.php');


JLoader::register('JFBConnectProfileGoogle', JPATH_SITE . '/components/com_jfbconnect/libraries/profiles/google.php');

class JFBConnectProviderGoogle extends JFBConnectProvider
{
    function __construct()
    {
        $this->name = "google";
        parent::__construct();

        $options = new JRegistry();
        $options->set('clientid', $this->appId);
        $options->set('clientsecret', $this->secretKey);
        $options->set('state', JSession::getFormToken()); // This is sent back to us to identify the provider

        // We want to use cURL, so need to build this all up. This also helps poke into each of the things when debugging
        $empty = new JRegistry();
        try
        {
            $transport = new JHttpTransportCurl($empty);
        }
        catch (Exception $e)
        {
            JFactory::getApplication()->enqueueMessage('JFBConnect requires the CURL PHP extension to be installed and callable.', 'error');
        }
        $http = new JHttp($options, $transport);

        $this->client = new JOAuth2Client($options, $http);
        $auth = new JGoogleAuthOauth2($options, $this->client);
        $this->provider = new JGoogle($options, $auth);

        // Refetch the Auth object based on G+
        $this->auth = $this->provider->data('plus');
        $this->auth->setOption('sendheaders', true); // Enabled for now. Should probably switch to force the redirect so we can just detect if user is logged in
        $this->auth->setOption('redirecturi', JURI::base() . 'index.php?option=com_jfbconnect&task=authenticate.callback&provider=google');

        // Setup Scope
        $this->auth->setOption('scope', $this->auth->getOption('scope', '') . " https://www.googleapis.com/auth/userinfo.email");
        if (JFBCFactory::config()->getSetting('google_openid_fallback'))
            $this->auth->setOption('scope', $this->auth->getOption('scope', '') . " email profile");

        $token = JFactory::getApplication()->getUserState('com_jfbconnect.' . $this->name . '.token', null);
        if ($token)
        {
            $token = (array)json_decode($token);
            $this->client->setToken($token);
        }

    }

    function getToken()
    {
        return $this->client->getToken();
    }

    function loginButton($params = null)
    {
        $googleLogin = "";
        if ($this->appId != "") // Basic check to make sure something is set and the Google Login has a chance of working
        {
            if (isset($params['buttonType']) && $params['buttonType'] == 'javascript')
            {
                $buttonSize = $params['buttonSize'];
                $renderKey = $this->getSocialTagRenderKey();
                $renderKeyStr = $renderKey != "" ? " key=" . $renderKey : "";
                return '{SCGoogleLogin size=' . $buttonSize . $renderKeyStr . '}';
            }
            else
                $googleLogin = $this->getLoginButtonWithImage($params, 'scGoogleLogin', 'sc_gologin');
        }

        return $googleLogin;
    }

    function connectButton($params)
    {
        $googleLogin = '';
        // This is ugly. Need a real call to get the data for a user
        $userData = JFBCFactory::usermap()->getUser(JFactory::getUser()->id, $this->name)->_data;
        if ($this->appId != "" && empty($userData->provider_user_id))
        {
            if ($params['buttonType'] == 'javascript')
            {
                $connectText = $params['buttonText'];
                $buttonSize = $params['buttonSize'];

                $buttonHtml = '<link rel="stylesheet" href="' . JURI::base() . 'components/com_jfbconnect/assets/sc_google.css" type="text/css" />';
                $buttonHtml .= '<div class="sc-google-connect-user">';
                $buttonHtml .= '<div class="scGoogleLogin"><a href="javascript:void(0)" onclick="jfbc.login.google();"><span class="scGoogleButton ' . $buttonSize . '"></span><span class="scGoogleLoginButton ' . $buttonSize . '">' . $connectText . '</span></a></div>';
                $buttonHtml .= '</div>';
                return $buttonHtml;
            }
            else
                $googleLogin = $this->getLoginButtonWithImage($params, 'scGoogleConnect', 'sc_goconnect');
        }

        return $googleLogin;
    }

    private function getLoginButtonWithImage($params, $loginClass, $loginID)
    {
        $buttonType = isset($params['buttonType']) ? $params['buttonType'] : "icon_text_button";
        $alignment = isset($params['alignment']) ? $params['alignment'] : "left";
        $orientation = isset($params['orientation']) ? $params['orientation'] : "";

        if ($buttonType == 'icon_text_button')
        {
            $display = ' class="show"';
            $googleLogin = $this->getImageButton(JURI::root() . 'media/sourcecoast/images/provider/button_google.png', $display, $alignment, $loginClass, $loginID);
        }
        else if ($buttonType == 'icon_button')
        {
            if ($orientation == 'side')
                $display = ' class="show"';
            else
                $display = '';
            $googleLogin = $this->getImageButton(JURI::root() . 'media/sourcecoast/images/provider/icon_google.png', $display, $alignment, $loginClass, $loginID);
        }
        else if ($buttonType == "image_link")
        {
            $linkImage = $params['goLinkImage'];
            $display = ' class="show"';
            $googleLogin = $this->getImageButton($linkImage, $display, $alignment, $loginClass, $loginID);
        }
        else
        {
            $googleLogin = '';
        }

        return $googleLogin;
    }

    private function getImageButton($buttonImage, $display, $alignment, $loginClass, $loginId)
    {
        return '<div class="' . $loginClass . ' pull-' . $alignment . '"><a' . $display . ' id="' . $loginId . '" href="javascript:void(0)" onclick="jfbc.login.google();"><img src="' . $buttonImage . '" /></a></div>';
    }

    function getSocialTagRenderKey()
    {
        return $this->configModel->getSetting('social_tag_admin_key');
    }

    /* getProviderUserId
    * Gets the provider User IdFacebook. This is regardless of whether they are mapped to an
    *  existing Joomla account.
    */
    function getProviderUserId()
    {
        if ($this->get('providerUserId', null) == null)
        {
            $profile = $this->profile->fetchProfile('me', 'id');
            $id = $profile->get('id');
            if (!empty($id))
                $this->set('providerUserId', $id);
            else
                $this->set('providerUserId', null);
        }
        return $this->get('providerUserId');
    }

    /* getMappedUserId
    * Gets the Google user id of user logged into Google if they have a usermapping to a Joomla user
    * Returns null if user is not mapped (or not logged into Google+).
    */
    function getMappedUserId()
    {
        $jUser = JFactory::getUser();
        if ($jUser->guest)
            return null;

        $mappedGoogleUserId = JFactory::getApplication()->getUserState('com_jfbconnect.google.mapped_user_id', null);
        if (!$mappedGoogleUserId)
        {
            $mappedGoogleUserId = $this->getProviderUserId();
            if (JFBCFactory::usermap()->getJoomlaUserId($mappedGoogleUserId, 'google') != $jUser->get('id'))
                $mappedGoogleUserId = null;

            JFactory::getApplication()->setUserState('com_jfbconnect.google.mapped_user_id', $mappedGoogleUserId);
        }
        return $mappedGoogleUserId;
    }

}