<?php
/**
 * @package        JFBConnect
 * @copyright (C) 2009-2013 by Source Coast - All rights reserved
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.event.plugin');
jimport('sourcecoast.openGraph');
jimport('sourcecoast.utilities');
jimport('sourcecoast.easyTags');
jimport('joomla.filesystem.file');

class plgSystemJFBCSystem extends JPlugin
{
    var $tagsToReplace = array(
        'jfbclogin' => 'getJFBCLogin',
        'jfbclike' => 'getJFBCLike',
        'jfbcsend' => 'getJFBCSend',
        'jfbccomments' => 'getJFBCComments',
        'jfbccommentscount' => 'getJFBCCommentsCount',
        'jfbcfan' => 'getJFBCFan',
        'jfbcfeed' => 'getJFBCFeed',
        'jfbcfollow' => 'getJFBCFollow',
        'jfbcfriends' => 'getJFBCFriends',
        'jfbcrecommendations' => 'getJFBCRecommendations',
        'jfbcrecommendationsbar' => 'getJFBCRecommendationsBar',
        'jfbcrequest' => 'getJFBCRequest',
        'jfbcsharedactivity' => 'getJFBCSharedActivity',
        "jfbcsharedialog" => 'getJFBCShareDialog',
        'jfbcsubscribe' => 'getJFBCSubscribe',
        "jfbcaction" => 'getJFBCAction',
        'scgooglelogin' => 'getSCGoogleLogin',
        "scgoogleplusone" => 'getSCGPlusOne',
        "sctwittershare" => 'getSCTwitterShare',
        "jlinkedshare" => 'getJLinkedShare',
        "scpinterest" => 'getSCPinterest'
    );

    var $metadataTagsToStrip = array('JFBC', 'JLinked', 'SCTwitterShare', 'SCGooglePlusOne');

    static $cssIncluded = false;

    function __construct(& $subject, $config)
    {
        $factoryFile = JPATH_ROOT . '/components/com_jfbconnect/libraries/factory.php';
        if (!JFile::exists($factoryFile))
        {
            JFactory::getApplication()->enqueueMessage("File missing: " . $factoryFile . "<br/>Please re-install JFBConnect or disable the JFBCSystem Plugin", 'error');
            return; // Don't finish loading this plugin to prevent other errors
        }
        require_once($factoryFile);
        // Need to load this as some custom developers expect this file to already be loaded and using the old JFBCFacebookLibrary classname
        // Doing this for backward compatibility in v5.1. Remove in the future
        require_once(JPATH_SITE . '/components/com_jfbconnect/libraries/provider/facebook.php');

        parent::__construct($subject, $config);
    }

    public function onAfterInitialise()
    {
        $app = JFactory::getApplication();
        if (!$app->isAdmin())
        {
            // Need to disable Page caching so that values fetched from Facebook are not saved for the next user!
            // Do this by setting the request type to POST. In the Cache plugin, it's checked for "GET". can't be that.
            $option = JRequest::getCmd("option");
            $view = JRequest::getCmd("view");
            if ($option == 'com_jfbconnect' && $view == 'loginregister')
                $_SERVER['REQUEST_METHOD'] = 'POST';

            // Need to load our plugin group early to be able to hook into to every step after
            JPluginHelper::importPlugin('opengraph');
            JPluginHelper::importPlugin('socialprofiles');

            $providers = JFBCFactory::getAllProviders();
            foreach ($providers as $provider)
                $provider->onAfterInitialise();
        }
    }

    public function onAfterRoute()
    {
        $app = JFactory::getApplication();
        if (!$app->isAdmin())
        {
            $app = JFactory::getApplication();
            $app->triggerEvent('onOpenGraphAfterRoute');
        }
    }

    // Called after the component has executed and it's output is available in the buffer
    // Modules have *not* executed yet
    public function onAfterDispatch()
    {
        $app = JFactory::getApplication();
        if (!$app->isAdmin())
        {
            $providers = JFBCFactory::getAllProviders();
            foreach ($providers as $provider)
                $provider->onAfterDispatch();

            foreach ($this->metadataTagsToStrip as $metadataTag)
            {
                $this->replaceTagInMetadata($metadataTag);
            }

            $doc = JFactory::getDocument();
            if ($doc->getType() == 'html')
            {
                $doc->addCustomTag('<JFBConnectSCTwitterJSPlaceholder />');
                $doc->addCustomTag('<JFBConnectSCGooglePlusOneJSPlaceholder />');
                $doc->addCustomTag('<JFBConnectSCPinterestJSPlaceholder />');
                $doc->addCustomTag('<JLinkedJfbcJSPlaceholder />');
                /*
                    if (JFBCFactory::config()->getSetting('jquery_load'))
                        $doc->addScript(JURI::base(true) . '/media/sourcecoast/js/jquery-1.8.3.min.js');
                    if (JFBCFactory::config()->getSetting('bootstrap_load'))
                    {
                        // JomSocial loads their own.. let's not trample
                        if (!(JFactory::getApplication()->input->getCmd('option') == 'com_community' &&
                                JFile::exists(JPATH_SITE . '/components/com_community/assets/bootstrap/bootstrap.min.js'))
                        )
                            $doc->addScript(JURI::base(true) . '/media/sourcecoast/js/bootstrap.min.js');
                    }
                 //SC16
                 //SC30*/
                if (JFBCFactory::config()->getSetting('jquery_load'))
                    $doc->addScript(JURI::base(true) . '/media/sourcecoast/js/jq-bootstrap-1.8.3.js');
            }

            //Add Login with FB button to com_users login view and mod_login
            if (JFBCFactory::config()->getSetting('show_login_with_joomla_reg'))
            {
                $renderKey = SCSocialUtilities::getJFBConnectRenderKey();
                $loginTag = '{JFBCLogin logout=false' . $renderKey . '}';
                SCEasyTags::extendJoomlaUserForms($loginTag);
            }

            // Add the Open Graph links to the user edit form.
            if ($this->showOpenGraphProfileLinks() && JFBCFactory::provider('facebook')->userIsConnected())
            {
                SCStringUtilities::loadLanguage('com_jfbconnect');

                $htmlTag = '<a href="' . JRoute::_('index.php?option=com_jfbconnect&view=opengraph&layout=activity') . '">' . JText::_('COM_JFBCONNECT_TIMELINE_ACTIVITY_LINK') . '</a>';
                $htmlTag .= '<br/><a href="' . JRoute::_('index.php?option=com_jfbconnect&view=opengraph&layout=settings') . '">' . JText::_('COM_JFBCONNECT_TIMELINE_CHANGESETTINGS') . '</a>';

                SCEasyTags::extendJoomlaUserForms($htmlTag, 'profile', true);
            }

            JPluginHelper::importPlugin('opengraph');
            $app->triggerEvent('onOpenGraphAfterDispatch');
        }
    }

    // Called right before the page is rendered
    public function onBeforeRender()
    {
    }

    public function onAfterRender()
    {
        $app = JFactory::getApplication();
        if (!$app->isAdmin())
        {
            $providers = JFBCFactory::getAllProviders();
            foreach ($providers as $provider)
                $provider->onAfterRender();

            $this->doTagReplacements();
        }
        return true;
    }

    private function showOpenGraphProfileLinks()
    {
        $db = JFactory::getDbo();
        $db->setQuery('SELECT COUNT(*) FROM #__opengraph_action WHERE published = 1');
        $numOGActionsEnabled = $db->loadResult();

        $user = JFactory::getUser();
        $db->setQuery('SELECT COUNT(*) FROM #__opengraph_activity WHERE user_id = ' . $user->id . ' AND status=1');
        $numActivities = $db->loadResult();

        return ($numOGActionsEnabled > 0) || ($numActivities > 0);
    }

    private function replaceTagInMetadata($metadataTag)
    {
        $doc = JFactory::getDocument();
        $description = $doc->getDescription();
        $replace = SCSocialUtilities::stripSystemTags($description, $metadataTag);

        if ($replace)
        {
            $description = SCStringUtilities::trimNBSP($description);
            $doc->setDescription($description);
        }
    }

    private function doTagReplacements()
    {
        $twitterTagFound = false;
        $googlePlusTagFound = false;
        $jLinkedTagFound = false;
        $pinterestTagFound = false;

        $tagsFound = false;
        $tagKeys = array_keys($this->tagsToReplace);
        foreach ($tagKeys as $tag)
        {
            $lowercaseTag = strtolower($tag);

            /*
             * Code to strip any {JFBCxyz} tags from head.
             */
            //Get the head
            $content = JResponse::getBody();
            $regex = '|<head(.*)?</head>|sui';
            if (preg_match($regex, $content, $matches))
            {
                if (count($matches) == 2) // more than one head is a problem, don't do anything
                {
                    //Remove the tag if it's in the head
                    $newHead = preg_replace('|{' . $tag . '(.*?)}|ui', '', $matches[0], -1, $count);

                    if ($count > 0)
                    {
                        //Replace the head
                        $content = preg_replace('|<head(.*)?</head>|sui', $newHead, $content, -1, $count);
                        if ($count == 1) // Only update the body if exactly one head was found and replaced
                        JResponse::setBody($content);
                    }
                }
            }

            //Tag has passed in values
            $regex = '/\{' . $tag . '\s+(.*?)\}/i';
            $currentTag1Found = $this->replaceTag($lowercaseTag, $regex);
            $tagsFound = $currentTag1Found || $tagsFound;

            //Tag with no values
            $regex = '/\{' . $tag . '}/i';
            $currentTag2Found = $this->replaceTag($lowercaseTag, $regex);
            $tagsFound = $currentTag2Found || $tagsFound;

            if ($currentTag1Found || $currentTag2Found)
            {
                if ($lowercaseTag == 'sctwittershare')
                    $twitterTagFound = true;
                else if ($lowercaseTag == 'scgoogleplusone')
                    $googlePlusTagFound = true;
                else if ($lowercaseTag == 'jlinkedshare')
                    $jLinkedTagFound = true;
                else if ($lowercaseTag == 'scpinterest')
                    $pinterestTagFound = true;
            }
        }
        $this->replaceGraphTags();
        $this->replaceJSPlaceholders($twitterTagFound, $googlePlusTagFound, $jLinkedTagFound, $pinterestTagFound);
    }

    private function replaceJSPlaceholders($twitterTagFound, $googlePlusTagFound, $jLinkedTagFound, $pinterestTagFound)
    {
        $uri = JURI::getInstance();
        $scheme = $uri->getScheme();

        //Twitter
        $twitterPlaceholder = '<JFBConnectSCTwitterJSPlaceholder />';
        if ($twitterTagFound)
            $twitterJavascript = '<script src="' . $scheme . '://platform.twitter.com/widgets.js"></script>';
        else
            $twitterJavascript = '';

        //GooglePlus
        $googlePlaceholder = '<JFBConnectSCGooglePlusOneJSPlaceholder />';
        if ($googlePlusTagFound)
        {
            $googleJavascript = "<script type=\"text/javascript\">
              (function() {
                var po = document.createElement('script'); po.type = 'text/javascript'; po.async = true;
                po.src = '" . $scheme . "://apis.google.com/js/plusone.js';
                var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(po, s);
              })();
            </script>";
        }
        else
            $googleJavascript = '';

        //Pinterest
        $pinterestPlaceholder = '<JFBConnectSCPinterestJSPlaceholder />';
        if ($pinterestTagFound)
        {
            $pinterestJavascript =
                    <<<EOT
                            <script type="text/javascript">
(function(d){
  var f = d.getElementsByTagName('SCRIPT')[0], p = d.createElement('SCRIPT');
  p.type = 'text/javascript';
  p.async = true;
  p.src = '//assets.pinterest.com/js/pinit.js';
  f.parentNode.insertBefore(p, f);
}(document));
</script>
EOT;
        }
        else
            $pinterestJavascript = '';

        //JLinked
        $jLinkedPlaceholder = '<JLinkedJfbcJSPlaceholder />';
        $jLinkedEnabled = SCSocialUtilities::isJLinkedInstalled() && SCSocialUtilities::areJLinkedTagsEnabled();
        if ($jLinkedTagFound && !$jLinkedEnabled)
        {
            $jLinkedJavascript = '<script src="' . $scheme . '://platform.linkedin.com/in.js"></script>';
            if ($scheme == 'https')
            {
                $jLinkedJavascript .= '<script type="text/javascript">' .
                        "IN.Event.on(IN,'frameworkLoaded',function(){if(/^https:\/\//i.test(location.href)){IN.ENV.images.sprite='https://www.linkedin.com/scds/common/u/img/sprite/'+IN.ENV.images.sprite.split('/').pop()}});
                    </script>";
            }
        }
        else
            $jLinkedJavascript = '';

        //Replace placeholder with Javascript if needed
        $contents = JResponse::getBody();
        $contents = str_replace($twitterPlaceholder, $twitterJavascript, $contents);
        $contents = str_replace($googlePlaceholder, $googleJavascript, $contents);
        $contents = str_replace($pinterestPlaceholder, $pinterestJavascript, $contents);
        $contents = str_replace($jLinkedPlaceholder, $jLinkedJavascript, $contents);
        JResponse::setBody($contents);
    }

    private function replaceTag($method, $regex)
    {
        $replace = FALSE;
        $contents = JResponse::getBody();
        if (preg_match_all($regex, $contents, $matches, PREG_SET_ORDER))
        {
            $count = count($matches[0]);
            if ($count == 0)
                return true;

            $jfbcRenderKey = SCSocialUtilities::getJFBConnectRenderKeySetting();

            foreach ($matches as $match)
            {
                if (isset($match[1]))
                    $val = $match[1];
                else
                    $val = '';

                $cannotRender = SCEasyTags::cannotRenderEasyTag($val, $jfbcRenderKey);
                if ($cannotRender)
                    continue;

                if (array_key_exists($method, $this->tagsToReplace))
                {
                    $methodName = $this->tagsToReplace[$method];
                    $newText = call_user_func(array('SCEasyTags', $methodName), $val);
                    $replace = TRUE;

                    if (!self::$cssIncluded)
                    {
                        self::$cssIncluded = true;
                        $newText = '<link rel="stylesheet" href="' . JURI::base(false) . 'components/com_jfbconnect/assets/jfbconnect.css" type="text/css" />' . $newText;
                    }
                }
                else
                {
                    $newText = '';
                    $replace = FALSE;
                }

                $search = '/' . preg_quote($match[0], '/') . '/';
                $contents = preg_replace($search, $newText, $contents, 1);
            }
            if ($replace)
                JResponse::setBody($contents);
        }

        return $replace;
    }

    private function getGraphContents($regex, &$contents, &$newGraphTags)
    {
        if (preg_match_all($regex, $contents, $matches, PREG_SET_ORDER))
        {
            $count = count($matches[0]);
            if ($count == 0)
                return true;

            $jfbcRenderKey = SCSocialUtilities::getJFBConnectRenderKeySetting();
            $jLinkedRenderKey = SCSocialUtilities::getJLinkedRenderKeySetting();

            foreach ($matches as $match)
            {
                if (isset($match[1]))
                    $val = $match[1];
                else
                    $val = '';

                $cannotRenderJFBC = SCEasyTags::cannotRenderEasyTag($val, $jfbcRenderKey);
                $cannotRenderJLinked = SCEasyTags::cannotRenderEasyTag($val, $jLinkedRenderKey);

                if ($cannotRenderJFBC && $cannotRenderJLinked)
                    continue;

                $newGraphTags[] = $val;
                $contents = str_replace($match[0], '', $contents);
            }
        }
    }

    private function replaceGraphTags()
    {
        $placeholder = '<SCOpenGraphPlaceholder />';
        $regex1 = '/\{SCOpenGraph\s+(.*?)\}/i';
        $regex2 = '/\{JFBCGraph\s+(.*?)\}/i';

        $newGraphTags1 = array();
        $newGraphTags2 = array();

        $contents = JResponse::getBody();
        $this->getGraphContents($regex1, $contents, $newGraphTags1);
        $this->getGraphContents($regex2, $contents, $newGraphTags2);

        $newGraphTags = array_merge($newGraphTags1, $newGraphTags2);
        //Replace Placeholder with new Head tags
        $defaultGraphFields = JFBCFactory::config()->getSetting('social_graph_fields');
        $locale = JFBCFactory::provider('facebook')->getLocale();

        $openGraphLibrary = OpenGraphLibrary::getInstance();
        $openGraphLibrary->addOpenGraphEasyTags($newGraphTags);
        $openGraphLibrary->addDefaultSettingsTags($defaultGraphFields);
        $openGraphLibrary->addAutoGeneratedTags($locale);
        $graphTags = $openGraphLibrary->buildCompleteOpenGraphList();

        $contents = $openGraphLibrary->removeOverlappingTags($contents);
        $search = '/' . preg_quote($placeholder, '/') . '/';
        $graphTags = str_replace('$', '\$', $graphTags);
        $contents = preg_replace($search, $graphTags, $contents, 1);
        $contents = str_replace($placeholder, '', $contents); //If JLinked attempts to insert, ignore
        JResponse::setBody($contents);
    }
}
