<?php
/**
 * @package        JFBConnect
 * @copyright (C) 2009-2013 by Source Coast - All rights reserved
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('sourcecoast.articleContent');
jimport('sourcecoast.openGraphPlugin');

class plgOpenGraphJReviews extends OpenGraphPlugin
{
    protected function init()
    {
        $this->extensionName = "jReviews";
        $this->supportedComponents[] = 'com_content';
        $this->supportedComponents[] = 'com_jreviews';

        // Enable this setting if the setOpenGraph function should be called even if an object type hasn't been defined
        // Good for setting title, description, image, etc tags on pages that may not be defined as objects by the admin
        // For example, in content, those are good tags to have (for Like buttons) regardless of if Actions are setup for the page
        $this->setsDefaultTags = true;

        // Define all types of pages this component can create and would be 'objects'
        // jReviews doesn't have it's own objects, it uses Joomla content. So we just have special actions on that content.
        $this->addSupportedObject("Listing", "listing");

        // Add actions that aren't passive (commenting, voting, etc).
        // Things that trigger just by loading the page should not be defined here unless extra logic is required
        // ie. Don't define reading an article
        $this->addSupportedAction("Review", "review");
    }

    protected function findObjectType($queryVars)
    {
        // Setup Object type for page
        $view = array_key_exists('view', $queryVars) ? $queryVars['view'] : '';
        // We return a specific category match first, then go for a matching directory
        $bestObject = null;
        if ($view == 'article')
        {
            $objectTypes = $this->getObjects('listing');
            JModelLegacy::addIncludePath(JPATH_SITE . '/components/com_content/models');
            $contentModel = JModelLegacy::getInstance('Article', 'ContentModel');
            $item = $contentModel->getItem($queryVars['id']);
            $itemCatId = $item->catid;

            $this->db->setQuery("SELECT dirid FROM #__jreviews_categories WHERE id = " . $itemCatId);
            $itemDirId = $this->db->loadResult();

            foreach ($objectTypes as $object)
            {
                // check if directory is set, if so, check that. If not, check categories
                $dirId = $object->params->get('directory', 0);
                if ($dirId != 0)
                {
                    if ($itemDirId == $dirId)
                        $bestObject = $object;
                } else
                {
                    $objectCatId = $object->params->get('category', 0);
                    if ($itemCatId == $objectCatId)
                        return $object;
                }
            }
        }
        return $bestObject;
    }

    protected function setOpenGraphTags()
    {
        $desc_type = 'custom_desc_introwords'; //Note: meta is same as blank value, since system plugin attempts to generate from metadescription if no value is found
        $desc_length = '10';

        $view = JRequest::getCmd('view');

        if ($this->object)
        {
            $desc_type = $this->object->params->get('custom_desc_type');
            $desc_length = $this->object->params->get('custom_desc_length');
        }

        if ($view == 'article')
        {
            $contentModel = JModelLegacy::getInstance('Article', 'ContentModel');
            $article = $contentModel->getItem();
            $rawImages = $article->images;
            $rawImages = explode("\n", $rawImages);
            foreach ($rawImages as $image)
            {
                $link = explode("|", $image);
                if (count($link) > 1)
                    $this->addOpenGraphTag('image', JURI::base() . 'images/' . $link[0], false);
            }

            if ($desc_type == 'custom_desc_introwords')
                $desc = SCArticleContent::getFirstArticleText($article, $desc_length, SC_INTRO_WORDS);
            else if ($desc_type == 'custom_desc_introchars')
                $desc = SCArticleContent::getFirstArticleText($article, $desc_length, SC_INTRO_CHARS);
            $this->addOpenGraphTag('description', $desc, true);
        }
    }

    /************* DEFINED ACTIONS CALLS *******************/
    /*** jReviews doesn't exit during AJAX calls, which is great! So, we wait till it's processed the review
     * and then check if the review was posted by check the database
     * To do this, we're overriding the onOpenGraphAfterDispatch() call, but ensuring we still call parent when we're done
     */
    public function onOpenGraphAfterDispatch()
    {
        // Checking actions here since
        if (JRequest::getCmd('format') == 'raw') // looks like a Reviews ajax request
        {
            foreach ($this->supportedActions as $supportedAction)
            {
                $actions = $this->jfbcOgActionModel->getActionsOfType($this->pluginName, $supportedAction);
                foreach ($actions as $action)
                    $this->checkAction($action);
            }
        }
        parent::onOpenGraphAfterDispatch();
    }

    private function checkAction($action)
    {
        $data = JRequest::getVar('data');
        if ($data['action'] == '_save'  && $data['controller'] == 'reviews')
        {
            // get the article ID being reviewed:
            if (!array_key_exists('pid', $data['Review']))
                return;
            $id = $data['Review']['pid'];
            $filter = JFilterInput::getInstance();
            $aid = $filter->clean($id, "INT");

            $user = JFactory::getUser();
            $uid = $user->get('id');
            $this->db->setQuery("SELECT pid FROM #__jreviews_comments WHERE userid = " . $uid . " ORDER BY id DESC LIMIT 1");
            $lastReviewId = $this->db->loadResult();

            if ($aid == $lastReviewId)
            {
                // Last review was for the article currently being reviewed. Publish the action
                JModelLegacy::addIncludePath(JPATH_SITE . '/components/com_content/models');
                $contentModel = JModelLegacy::getInstance('Article', 'ContentModel');
                $article = $contentModel->getItem($aid);
                $article->slug = $article->id;

                $url = SCArticleContent::getCurrentURL($article, true);
                $this->triggerAction($action, $url);
            }
        }
    }
}